#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cptutils/svg-list.h"

#include <stdlib.h>

struct svg_list_t
{
  size_t n, alloc;
  svg_t *svg;
};

#define LIST_INI 20
#define LIST_INC 20

svg_list_t* svg_list_new(void)
{
  svg_list_t *list;

  if ((list = calloc(1, sizeof(svg_list_t))) != NULL)
    {
      svg_t *svg;

      if ((svg = calloc(LIST_INI, sizeof(svg_t))) != NULL)
	{
	  list->svg = svg;
	  list->alloc = LIST_INI;
	  list->n = 0;

	  return list;
	}

      free(list);
    }

  return NULL;
}

void svg_list_destroy(svg_list_t *list)
{
  if (list != NULL)
    {
      for (size_t i = 0 ; i < list->n ; i++)
        svg_deinit(list->svg + i);
      free(list->svg);
    }
  free(list);
}

int svg_list_iterate(svg_list_t *list,
                     int (*f)(svg_t*, void*),
                     void *opt)
{
  for (size_t i = 0 ; i < list->n ; i++)
    {
      int err;
      svg_t *svg = list->svg + i;

      if ((err = f(svg, opt)) != 0)
        return err;
    }

  return 0;
}

svg_t* svg_list_select(svg_list_t *list,
                       int (*f)(svg_t*, void*),
                       void *opt)
{
  for (size_t i = 0 ; i < list->n ; i++)
    {
      svg_t *svg = list->svg + i;

      if (f(svg, opt) != 0)
        return svg;
    }

  return NULL;
}

svg_t* svg_list_next(svg_list_t *list)
{
  size_t
    n = list->n,
    a = list->alloc;

  if (n < a)
    {
      list->n++;

      svg_t *svg = list->svg + n;

      if (svg_init(svg) == 0)
        return svg;
      else
        return NULL;
    }
  else
    {
      a += LIST_INC;

      svg_t *svg;

      if ((svg = realloc(list->svg, sizeof(svg_t) * a)) != NULL)
        {
          list->alloc = a;
          list->svg = svg;

          return svg_list_next(list);
        }
      else
        return NULL;
    }
}

int svg_list_revert(svg_list_t *list)
{
  size_t n = list->n;

  if (n == 0)
    return 1;

  svg_t *svg = &(list->svg[n - 1]);
  svg_deinit(svg);
  list->n--;

  return 0;
}

size_t svg_list_size(const svg_list_t *list)
{
  return list->n;
}

const svg_t* svg_list_entry(const svg_list_t *list, size_t i)
{
  if (i >= list->n)
    return NULL;
  return list->svg + i;
}
