#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cptutils/xml-id.h"

#include <stddef.h>
#include <stdbool.h>
#include <string.h>
#include <ctype.h>

static bool id_start_char(char c)
{
  return isalpha(c);
}

static bool id_mid_char(char c)
{
  if (isalnum(c))
    return true;

  /*
    note that the standard permits an underscore in an id,
    we delibrately do not, so those will be replaced by
    hyphens and make a more-consistent id
  */

  switch (c)
    {
    case '-':
    case '.':
    case ':':
      return true;
    default:
      return false;
    }
}

static int sanitise2(char *src, size_t n)
{
  /*
    any leading character which is not an id start character,
    replace with a hyphen
  */

  size_t i;

  for (i = 0 ; i < n ; i++)
    if (id_start_char(src[i]))
      break;
    else
      src[i] = '-';

  /*
    i is now the position of the first id start character, or
    i is n+1 if there are none, now replace all the rest of the
    chanraters which are not id middle characters by a hyphen
  */

  for (++i ; i < n ; i++)
    if (! id_mid_char(src[i]))
      src[i] = '-';

  /*
    compress hyphens, we keep 2 indices, i is the source, j the
    destination -- first the leading hyphens
  */

  for (i = 0 ; i < n ; i++)
    if (src[i] != '-')
      break;

  size_t j = 0;

  while (i < n)
    {
      src[j++] = src[i];
      if (src[i] == '-')
        while ((i < n) && (src[i] == '-'))
          i++;
      else
        i++;
    }

  /* may be a single hypehn at end */

  if ((j > 0) && (src[j - 1] == '-'))
    j--;

  src[j] = '\0';

  /* possibly we've nothing left, so error */

  if (j == 0)
    return 1;

  return 0;
}

char* xml_id_sanitise(const char *src)
{
  if (src == NULL)
    return NULL;

  size_t n = strlen(src);

  switch (n)
    {
    case 0:
      return NULL;
    case 1:
      if (! id_start_char(src[0]))
        return NULL;
      break;
    }

  char src2[n + 1];

  memcpy(src2, src, n + 1);

  if (sanitise2(src2, n) == 0)
    return strdup(src2);

  return NULL;
}
